[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Analytics Datastore client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)](https://developer.factset.com/api-catalog/analytics-datastore-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.AnalyticsDatastore/0.22.7)](https://pypi.org/project/fds.sdk.AnalyticsDatastore/v/0.22.7)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Allow clients to fetch precalculated Analytics through predeterministic URLs.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 0.22.7
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.AnalyticsDatastore==0.22.7
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.AnalyticsDatastore==0.22.7
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.AnalyticsDatastore
from fds.sdk.AnalyticsDatastore.api import documents_api
from fds.sdk.AnalyticsDatastore.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.AnalyticsDatastore.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.AnalyticsDatastore.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.AnalyticsDatastore.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = documents_api.DocumentsApi(api_client)
    document_request = DocumentRequest(
        meta=MetaRequestModel(
            pagination=MetaRequestModelPagination(
                offset=0,
                limit=10,
            ),
        ),
        data=DocumentRequestData(
            path="Client:/Publisher/TEST_ACCOUNT.ACCT",
            product_type="Publisher",
            tags={
                "key": "key_example",
            },
        ),
    ) # DocumentRequest | Example request of a filtered Documents search call

    try:
        # Gets a list of Documents that a user has access to.
        # example passing only required values which don't have defaults set
        api_response = api_instance.get_documents_end_point(document_request)

        pprint(api_response)
    except fds.sdk.AnalyticsDatastore.ApiException as e:
        print("Exception when calling DocumentsApi->get_documents_end_point: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Gets a list of Documents that a user has access to.
    #     api_response, http_status_code, response_headers = api_instance.get_documents_end_point_with_http_info(document_request)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.AnalyticsDatastore.ApiException as e:
    #     print("Exception when calling DocumentsApi->get_documents_end_point: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Gets a list of Documents that a user has access to.
    #     async_result = api_instance.get_documents_end_point_async(document_request)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.AnalyticsDatastore.ApiException as e:
    #     print("Exception when calling DocumentsApi->get_documents_end_point: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Gets a list of Documents that a user has access to.
    #     async_result = api_instance.get_documents_end_point_with_http_info_async(document_request)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.AnalyticsDatastore.ApiException as e:
    #     print("Exception when calling DocumentsApi->get_documents_end_point: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.AnalyticsDatastore

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.AnalyticsDatastore.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.AnalyticsDatastore

configuration = fds.sdk.AnalyticsDatastore.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.AnalyticsDatastore

configuration = fds.sdk.AnalyticsDatastore.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.AnalyticsDatastore

configuration = fds.sdk.AnalyticsDatastore.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*DocumentsApi* | [**get_documents_end_point**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentsApi.md#get_documents_end_point) | **POST** /analytics/pub-datastore/tag-search/v1/documents/search | Gets a list of Documents that a user has access to.
*DocumentsApi* | [**get_product_types**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentsApi.md#get_product_types) | **POST** /analytics/pub-datastore/tag-search/v1/product-types/search | Gets a list of product types that a user has access to.
*DocumentsApi* | [**get_single_tag_end_point**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentsApi.md#get_single_tag_end_point) | **GET** /analytics/pub-datastore/tag-search/v1/tags/{name} | Get a single tag definition based on its name
*DocumentsApi* | [**get_tag_values_end_point**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentsApi.md#get_tag_values_end_point) | **POST** /analytics/pub-datastore/tag-search/v1/tags/{name}/values | Gets a list of tag values for the given name
*DocumentsApi* | [**get_tags_end_point**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentsApi.md#get_tags_end_point) | **GET** /analytics/pub-datastore/tag-search/v1/tags | Gets a list of tag definitions for the user.
*MappingsApi* | [**get_insight_mapping**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/MappingsApi.md#get_insight_mapping) | **GET** /analytics/pub-datastore/swivel/v1/{document}/{assetName}/{reportId}/{tileId}/{accountId}/{configId}/{date} | Redirects to the underlying Insight/Vault data in a Stach format.
*MappingsApi* | [**get_pdf_ppt_mapping**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/MappingsApi.md#get_pdf_ppt_mapping) | **GET** /analytics/pub-datastore/swivel/v1/{document}/{accountId}/{date} | Redirects to the raw PPT/PDF output created by the Publisher service.
*MappingsApi* | [**get_stach_mapping**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/MappingsApi.md#get_stach_mapping) | **GET** /analytics/pub-datastore/swivel/v1/{document}/{assetName}/{reportId}/{tileId}/{accountId}/{date} | Redirects to the underlying PA/SPAR data represented in a Stach format.
*MappingsApi* | [**get_stach_mapping_no_date**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/MappingsApi.md#get_stach_mapping_no_date) | **GET** /analytics/pub-datastore/swivel/v1/{document}/{assetName}/{reportId}/{tileId}/{accountId} | Redirects to the underlying PA/SPAR data represented in a Stach format, with no as of date specified.
*ObjectsApi* | [**get_cargo_endpoint**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/ObjectsApi.md#get_cargo_endpoint) | **GET** /analytics/pub-datastore/cargo/v1/groups/{groupId}/objects/{objectId} | Gets an object given an ID. In this case ID retrieved from mapping Location header, object data in response body.


## Documentation For Models

 - [DocumentListResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentListResponse.md)
 - [DocumentModel](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentModel.md)
 - [DocumentRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentRequest.md)
 - [DocumentRequestData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/DocumentRequestData.md)
 - [MetaModel](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/MetaModel.md)
 - [MetaModelPagination](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/MetaModelPagination.md)
 - [MetaRequestModel](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/MetaRequestModel.md)
 - [MetaRequestModelPagination](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/MetaRequestModelPagination.md)
 - [ProductTypeListResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/ProductTypeListResponse.md)
 - [StringOrNumberTagValue](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/StringOrNumberTagValue.md)
 - [TagFilterModel](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/TagFilterModel.md)
 - [TagFilterModelData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/TagFilterModelData.md)
 - [TagListResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/TagListResponse.md)
 - [TagModel](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/TagModel.md)
 - [TagResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/TagResponse.md)
 - [TagValuesListResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/AnalyticsDatastore/v1/docs/TagValuesListResponse.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.AnalyticsDatastore.apis and fds.sdk.AnalyticsDatastore.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.AnalyticsDatastore.api.default_api import DefaultApi`
- `from fds.sdk.AnalyticsDatastore.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.AnalyticsDatastore
from fds.sdk.AnalyticsDatastore.apis import *
from fds.sdk.AnalyticsDatastore.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

